// <file>
//     <copyright see="prj:///doc/copyright.txt"/>
//     <license see="prj:///doc/license.txt"/>
//     <owner name="Siegfried Pammer" email="sie_pam@gmx.at"/>
//     <version>$Revision$</version>
// </file>

using DreamShieldBinding;
using System;
using System.IO;
using System.Windows.Forms;
using System.Windows.Forms.Integration;
using ICSharpCode.Core.Presentation;
using ICSharpCode.SharpDevelop;
using ICSharpCode.SharpDevelop.Gui;

namespace ICSharpCode.Core.Presentation
{	
	/// <summary>
	/// Description of the view content
	/// </summary>
	public class WpfViewer : AbstractViewContent
	{
		SharpDevelopElementHost host;
		DreamShieldView dataView;
		string file;
		
		/// <summary>
		/// Returns the ProfilerView of this instance.
		/// </summary>
		public DreamShieldView DataView {
			get { return dataView; }
		}
		
		/// <summary>
		/// The <see cref="System.Windows.Forms.Control"/> representing the view
		/// </summary>
		public override Control Control {
			get {
				return this.host;
			}
		}
		
		public override string PrimaryFileName {
			get { return file; }
		}
		
		/// <summary>
		/// Creates a new WpfViewer object
		/// </summary>
		public WpfViewer(string file)
		{
			//this.Files.Add(file);
			this.file = file;
			this.TabPageText = "Design";
			this.TitleName = Path.GetFileName(file);
			this.host = new SharpDevelopElementHost(dataView = new DreamShieldView(file));
			// HACK : Make host.Child visible
			WorkbenchSingleton.SafeThreadAsyncCall(
				() => {
					this.host.Dock = DockStyle.None;
					this.host.Dock = DockStyle.Fill;
				}
			);
		}

		/// <summary>
		/// Refreshes the view
		/// </summary>
		public override void RedrawContent()
		{
			// TODO: Refresh the whole view control here, renew all resource strings
			//       Note that you do not need to recreate the control.
		}
		
		/// <summary>
		/// Cleans up all used resources
		/// </summary>
		public override void Dispose()
		{
			this.host.Dispose();
			base.Dispose();
		}
	}
}
